(load "env-dia.scm")

(env-dia 'flush)

;; Some simple procedures
(define (square x) (* x x))
(define (double x) (+ x x))

;; Test 2 pointers from same environment pointing to same
;; destination (doesn't work with GraphViz < 2.3)
(define double2 double)

;; A number binding
(define pi 3.1415)

;; A binding to a list
(define my-list '(1 2 3))

;; A binding with symbols requiring escaping
(define <my&sym> '>my&val<)

;; A procedure that generates procedures with persistent local state
;; All environments are retained
(define counter
  (let ((count 0))
    (lambda ()
      (set! count (+ 1 count))
      ; Extra call to make sure the current environment shows up in the 
      ; diagram that gets drawn.
      (env-dia 'retain (the-environment))      
      count)))

; Run the counter a few times
(counter) (counter) (counter) (counter) (counter)

;; Now let's not keep the extra environments
(define counter
  (let ((count 0))
    (lambda ()
      (set! count (+ 1 count))
      count)))

; Run the counter a few times
(counter) (counter) (counter) 

; Show two bindings to the car implementation
; (demonstrates compiled procs, multiple incoming arrows, unsuppress-name)
(env-dia 'debug-on)
(env-dia 'unsuppress-name 'max user-initial-environment)
(define my-max max)
(env-dia 'debug-off)

; Primitives
(define my-car car)

; Suppress these bindings
(define should-not-be-visible 1)
(env-dia 'suppress-name 'should-not-be-visible (the-environment))
(env-dia 'suppress-name 'should-not-be-visible-either (the-environment))
(define should-not-be-visible-either 1)
(define should-not-be-visible-either 1)

;; Draw diagrams
(env-dia 'render "env-dia-test")
